/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert ein Objekt vom Typ 'typ.sswSbaUmfeldDatenSensor'.<p>
 * 
 * Die Klasse dient zur Verwaltung von Daten eines Umfelddatensensors.
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class UmfeldDatenSensor implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private Debug m_debug = Debug.getLogger();
	/**
	 * Aktuelle Verbindung zum Datenverteiler.
	 */
	private ClientDavConnection m_connection;
	/**
	 * Zugriff auf das aktuelle Datenmodell.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des Messquerschnitt.
	 */
	private SystemObject m_objekt;
	/**
	 * Attributgruppe fr den Datenempfang Sichtweitenstufe.
	 */
	private AttributeGroup m_atgUfdsSichtWeite;
	/**
	 * Aspekt fr den Datenempfang der Sichtweitenstufe.
	 */
	private Aspect m_aspKlassifizierung;
	/**
	 * Aktuelle Sichtstufe des Umfelddatensensor.
	 */
	private int m_sichtStufe;
	
	/**
	 * Konstruktor mit bernahme der Parameter.<p>
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param objekt Systemobjekt Messquerschnitt.
	 */
	public UmfeldDatenSensor(ClientDavInterface connection, SystemObject objekt)
	{
		m_connection = (ClientDavConnection)connection;
		m_dataModel = m_connection.getDataModel();
		
		m_objekt = objekt;

		m_atgUfdsSichtWeite = m_dataModel.getAttributeGroup("atg.ufdsStufeSichtWeite");		
		m_aspKlassifizierung = m_dataModel.getAspect("asp.klassifizierung");
	}

	/**
	 * Anmelden der angemeldeten Daten am Datenverteiler.
	 */
	public void datenDavAnmelden()
	{
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgUfdsSichtWeite, m_aspKlassifizierung);
	}
	/**
	 * Abmelden der angemeldeten Daten am Datenverteiler.
	 */
	public void datenDavAbmelden()
	{
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgUfdsSichtWeite, m_aspKlassifizierung);
	}
	
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
    try
    {  
    	dd = new DataDescription(atg, asp);

    	m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        
    	ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAbmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAbmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
  	try
		{
			dd = new DataDescription(atg, asp);

			m_connection.unsubscribeReceiver(instanz, objekt, dd);
			  
			ok = true;
		}
		catch (Exception e)
		{
			m_debug.error(("Daten konnten nicht abgemeldet werden " + e.getMessage()));
		}
    
    return (ok);
  }

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData[] results)
	{
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
			// Sichtweite
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgUfdsSichtWeite))
    	{ 
    		System.out.println("Sichtstufe -> " + resultData);

    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			m_sichtStufe = data.getUnscaledValue("Stufe").intValue();
    		}
    		else
    		{    			
    			m_sichtStufe = -1; 
    		}

    	}
		}		
	}

	/**
	 * Gibt die aktuelle Sichtstufe zurck.
	 * @return aktuelle Sichtstufe.
	 */
	public int getSichtStufe()
	{
		return m_sichtStufe;
	}

	/**
	 * Gibt das Objekt zurck.
	 * @return Objekt.
	 */
	public SystemObject getObjekt()
	{
		return m_objekt;
	}

	/**
	 * Gibt die Pid des Objekts zurck.
	 * @return Pid des Objekts.
	 */
	public String getPid()
	{
		if (m_objekt != null)
			return m_objekt.getPid();
		return null;
	}
}
